@@ -18,13 +18,22 @@
     You should have received a copy of the GNU General Public License
     along with MRtrix.  If not, see <http://www.gnu.org/licenses/>.
 
+    14-07-2010 Robert E. Smith <r.smith@brain.org.au>
+    * major overhaul of tracks2prob (converted from mrtrix development branch)
+
     04-08-2010 Robert E. Smith <r.smith@brain.org.au>
     * reverted to previous interpretation of voxel values as the number
       of tracks detected as entering each voxel, without normalisation
 
-    24-08-2010 Robert Smith <r.smith@brain.org.au>
+    24-08-2010 Robert E. Smith <r.smith@brain.org.au>
     * fix bug in handling of multiple contributions to a single voxel from a single track
 
+    19-12-2011 Robert E. Smith <r.smith@brain.org.au>
+    * added capabilty to generate length-scaled TDI
+
+    12-01-2012 Robert E. Smith <r.smith@brain.org.au>
+    * tdi calculated using either floating-point or integer buffer depending on user options
+
 */
 
 
@@ -35,7 +44,7 @@
 #include "dwi/tractography/properties.h"
 
 #include <set>
-
+#include <stdint.h>
 
 
 const gchar* data_type_choices[] = { "FLOAT32", "FLOAT32LE", "FLOAT32BE", "FLOAT64", "FLOAT64LE", "FLOAT64BE",
@@ -77,6 +86,8 @@ OPTIONS = {
 
   Option ("fraction", "output fibre fraction", "produce an image of the fraction of fibres through each voxel (as a proportion of the total number in the file), rather than the count."),
 
+  Option ("lstdi", "length-scaled TDI", "scale the contribution of each track to the density image by the inverse of the streamline length"),
+
   Option ("datatype", "data type", "specify output image data type.")
     .append (Argument ("spec", "specifier", "the data type specifier.").type_choice (data_type_choices)),
 
@@ -108,7 +119,7 @@ class Voxel
   public:
     Voxel () : x (0), y (0), z (0) { }
 
-    Voxel (const size_t _x_, const size_t _y_, const size_t _z_) :
+    Voxel (const int _x_, const int _y_, const int _z_) :
       x (_x_),
       y (_y_),
       z (_z_) { }
@@ -141,10 +152,10 @@ class Voxel
 
     bool test_bounds (const Image::Header& H) const 
     {
-      return (x < size_t(H.dim(0)) && y < size_t(H.dim(1)) && z < size_t(H.dim(2)));
+      return (x >= 0 && x < H.dim(0) && y >= 0 && y < H.dim(1) && z >= 0 && z < H.dim(2));
     }
 
-    size_t x, y, z;
+    int x, y, z;
 
 };
 
@@ -176,7 +187,8 @@ class VoxelDir : public Voxel
 };
 
 
-
+class SetVoxel    : public std::set     <Voxel>    { public: unsigned int length; };
+class SetVoxelDir : public std::multiset<VoxelDir> { public: unsigned int length; };
 
 
 
@@ -185,19 +197,19 @@ class Resampler
 {
 
   public:
-    Resampler (const Math::Matrix& interp_matrix, const size_t c) :
+    Resampler (const Math::Matrix& interp_matrix, const unsigned int c) :
       M (interp_matrix),
       columns (c),
       data (4, c) { }
 
     ~Resampler() { }
 
-    size_t get_columns () const { return (columns); }
+    unsigned int get_columns () const { return (columns); }
     bool valid () const { return (M.is_valid()); }
 
     void init (const float* a, const float* b, const float* c)
     {
-      for (size_t i = 0; i != columns; ++i) {
+      for (unsigned int i = 0; i != columns; ++i) {
         data(0,i) = 0.0;
         data(1,i) = a[i];
         data(2,i) = b[i];
@@ -207,7 +219,7 @@ class Resampler
 
     void increment (const float* a)
     {
-      for (size_t i = 0; i != columns; ++i) {
+      for (unsigned int i = 0; i != columns; ++i) {
         data(0,i) = data(1,i);
         data(1,i) = data(2,i);
         data(2,i) = data(3,i);
@@ -222,7 +234,7 @@ class Resampler
 
   private:
     const Math::Matrix& M;
-    size_t columns;
+    const unsigned int columns;
     Math::Matrix data;
 
 };
@@ -241,7 +253,7 @@ template <class T> class TrackMapper
       resample_matrix (interp_matrix),
       R (resample_matrix, 3) { }
 
-    void map (std::vector<Point>& tck, std::set<T>& output)
+    void map (std::vector<Point>& tck, T& output)
     {
       Math::Matrix data;
       if (R.valid()) {
@@ -263,7 +275,7 @@ template <class T> class TrackMapper
 
     void tck_interp_prepare (std::vector<Point>& v)
     {
-      const size_t s = v.size();
+      const unsigned int s = v.size();
       v.insert    (v.begin(), v[ 0 ] + (2 * (v[ 0 ] - v[ 1 ])) - (v[ 1 ] - v[ 2 ]));
       v.push_back (           v[ s ] + (2 * (v[ s ] - v[s-1])) - (v[s-1] - v[s-2]));
     }
@@ -276,33 +288,34 @@ template <class T> class TrackMapper
       std::vector<Point> out;
       tck_interp_prepare (tck);
       R.init (tck[0].get(), tck[1].get(), tck[2].get());
-      for (size_t i = 3; i < tck.size(); ++i) {
+      for (unsigned int i = 3; i < tck.size(); ++i) {
         out.push_back (tck[i-2]);
         R.increment (tck[i].get());
         R.interpolate (data);
-        for (size_t row = 0; row != data.rows(); ++row)
+        for (unsigned int row = 0; row != data.rows(); ++row)
           out.push_back (Point (data(row,0), data(row,1), data(row,2)));
       }
       out.push_back (tck[tck.size() - 2]);
       out.swap (tck);
     }
 
-    void voxelise (const std::vector<Point>& tck, std::set<T>& voxels) const;
+    void voxelise (const std::vector<Point>& tck, T& voxels) const;
 
 };
 
 
-template<> void TrackMapper<Voxel>::voxelise (const std::vector<Point>& tck, std::set<Voxel>& voxels) const
+template<> void TrackMapper<SetVoxel>::voxelise (const std::vector<Point>& tck, SetVoxel& voxels) const
 {
   for (std::vector<Point>::const_iterator i = tck.begin(); i != tck.end(); ++i) {
     Voxel vox (interp.R2P (*i));
     if (vox.test_bounds (H)) 
       voxels.insert (vox);
   }
+  voxels.length = tck.size() - 1;
 }
 
 
-template<> void TrackMapper<VoxelDir>::voxelise (const std::vector<Point>& tck, std::set<VoxelDir>& voxels) const
+template<> void TrackMapper<SetVoxelDir>::voxelise (const std::vector<Point>& tck, SetVoxelDir& voxels) const
 {
   std::vector<Point>::const_iterator prev = tck.begin();
   const std::vector<Point>::const_iterator last = tck.end() - 1;
@@ -320,6 +333,7 @@ template<> void TrackMapper<VoxelDir>::voxelise (const std::vector<Point>& tck,
     vox.set_dir ((*last - *prev).normalise());
     voxels.insert (vox);
   }
+  voxels.length = tck.size() - 1;
 }
 
 
@@ -332,23 +346,24 @@ template <class T> class MapWriterBase
 {
 
   public:
-    MapWriterBase (Image::Position& p, const float fraction_scaling_factor) :
+    MapWriterBase (Image::Position& p, const float fraction_scaling_factor, const bool length_scaled) :
       pos (p),
       H (p.image.header()),
-      scale (fraction_scaling_factor)
-    {
-      buffer_size = H.dim(0) * H.dim(1) * H.dim(2);
-    }
+      scale (fraction_scaling_factor),
+      lstdi (length_scaled),
+      buffer_size (H.dim(0) * H.dim(1) * H.dim(2)) { }
 
-    virtual void write (const std::set<T>& voxels) = 0;
+    virtual void write (const T& voxels) = 0;
 
    protected:
     Image::Position& pos;
     const Image::Header& H;
-    float scale;
-    size_t buffer_size;
+    const float scale;
+    const bool lstdi;
+    const size_t buffer_size;
 
-    size_t voxel_to_index(const T& vox)
+    template <class V>
+    size_t voxel_to_index (const V& vox)
     {
       return (vox.x + (vox.y * H.dim(0)) + (vox.z * H.dim(0) * H.dim(1)));
     }
@@ -356,50 +371,89 @@ template <class T> class MapWriterBase
 };
 
 
-class MapWriter : public MapWriterBase<Voxel> 
+template <class value_type>
+class MapWriter : public MapWriterBase<SetVoxel>
 {
 
   public:
-    MapWriter (Image::Position& p, const float fraction_scaling_factor) :
-      MapWriterBase<Voxel> (p, fraction_scaling_factor),
-      buffer (new size_t[buffer_size])
+    MapWriter (Image::Position& p, const float fraction_scaling_factor, const bool length_scaled) :
+      MapWriterBase<SetVoxel> (p, fraction_scaling_factor, length_scaled),
+      buffer (new value_type[buffer_size])
     {
-      memset(buffer, 0, buffer_size * sizeof(size_t));
+      memset(buffer, 0, buffer_size * sizeof(value_type));
     }
 
-    ~MapWriter ()
-    {
-      ProgressBar::init(H.dim(2), "writing image... ");
-      size_t index = 0;
-      for (pos.set(2,0); pos[2] < H.dim(2); pos.inc(2)) {
-        for (pos.set(1,0); pos[1] < H.dim(1); pos.inc(1)) {
-          for (pos.set(0,0); pos[0] < H.dim(0); pos.inc(0), ++index)
-            pos.value (scale * buffer[index]);
-        }
-        ProgressBar::inc();
-      }
-      ProgressBar::done();
-      delete[] buffer;
-    }
+    ~MapWriter();
 
-    void write (const std::set<Voxel>& voxels)
-    {
-      for (std::set<Voxel>::const_iterator i = voxels.begin(); i != voxels.end(); ++i)
-        buffer[voxel_to_index(*i)] += 1;
-    }
+    void write (const SetVoxel&);
 
   private:
-    size_t* buffer;
+    value_type* buffer;
 
 };
 
 
-class MapWriterColour : public MapWriterBase<VoxelDir>
+template <>
+MapWriter<uint32_t>::~MapWriter<uint32_t> ()
+{
+
+  ProgressBar::init (H.dim(2), "writing image... ");
+  size_t index = 0;
+  for (pos.set(2,0); pos[2] < H.dim(2); pos.inc(2)) {
+    for (pos.set(1,0); pos[1] < H.dim(1); pos.inc(1)) {
+      for (pos.set(0,0); pos[0] < H.dim(0); pos.inc(0), ++index)
+        pos.value (buffer[index]);
+    }
+    ProgressBar::inc();
+  }
+  ProgressBar::done();
+  delete[] buffer;
+
+}
+
+template <>
+MapWriter<float>::~MapWriter<float> ()
+{
+
+  ProgressBar::init (H.dim(2), "writing image... ");
+  size_t index = 0;
+  for (pos.set(2,0); pos[2] < H.dim(2); pos.inc(2)) {
+    for (pos.set(1,0); pos[1] < H.dim(1); pos.inc(1)) {
+      for (pos.set(0,0); pos[0] < H.dim(0); pos.inc(0), ++index)
+        pos.value (scale * buffer[index]);
+    }
+    ProgressBar::inc();
+  }
+  ProgressBar::done();
+  delete[] buffer;
+
+}
+
+
+
+template <>
+void MapWriter<uint32_t>::write (const SetVoxel& voxels)
+{
+  for (SetVoxel::const_iterator i = voxels.begin(); i != voxels.end(); ++i)
+    ++buffer[voxel_to_index(*i)];
+}
+
+template <>
+void MapWriter<float>::write (const SetVoxel& voxels)
+{
+  for (SetVoxel::const_iterator i = voxels.begin(); i != voxels.end(); ++i)
+    buffer[voxel_to_index(*i)] += lstdi ? (1.0 / float(voxels.length)) : 1.0;
+}
+
+
+
+
+class MapWriterColour : public MapWriterBase<SetVoxelDir>
 {
 
   public:
-    MapWriterColour(Image::Position& p, const float fraction_scaling_factor) :
-      MapWriterBase<VoxelDir> (p, fraction_scaling_factor),
+    MapWriterColour (Image::Position& p, const float fraction_scaling_factor, const bool length_scaled) :
+      MapWriterBase<SetVoxelDir> (p, fraction_scaling_factor, length_scaled),
       buffer (new Point[buffer_size])
     {
       for (size_t i = 0; i != buffer_size; ++i)
@@ -424,16 +478,16 @@ class MapWriterColour : public MapWriterBase<VoxelDir>
       delete[] buffer;
     }
 
-    void write (const std::set<VoxelDir>& voxels)
+    void write (const SetVoxelDir& voxels)
     {
-      std::set<VoxelDir>::const_iterator i = voxels.begin();
+      SetVoxelDir::const_iterator i = voxels.begin();
       while (i != voxels.end()) {
         const std::set<VoxelDir>::const_iterator this_voxel = i;
-        Point this_voxel_dir(0.0, 0.0, 0.0);
+        Point this_voxel_dir (0.0, 0.0, 0.0);
         do {
           this_voxel_dir += i->dir;
         } while ((++i) != voxels.end() && *i == *this_voxel);
-        buffer[voxel_to_index(*this_voxel)] += this_voxel_dir.normalise();
+        buffer[voxel_to_index(*this_voxel)] += this_voxel_dir.normalise() * (lstdi ? (1.0 / float(voxels.length)) : 1.0);
       }
     }
 
@@ -463,7 +517,7 @@ class Hermite
       w[3] = (0.5- t) * (p3 - p2);
     }
 
-    float coef (size_t i) const { return (w[i]); }
+    float coef (const unsigned int i) const { return (w[i]); }
 
     float value (const float& a, const float& b, const float& c, const float& d) const
     {
@@ -512,7 +566,7 @@ void generate_header (Image::Header& header, DWI::Tractography::Reader& file, co
   header.name = "tckmap image header";
   header.axes.set_ndim(3);
 
-  for (size_t i = 0; i != 3; ++i) {
+  for (unsigned int i = 0; i != 3; ++i) {
     header.axes.dim[i]    = ceil ((max_values[i] - min_values[i]) / voxel_size[i]);
     header.axes.vox[i]    = voxel_size[i];
     header.axes.axis[i]   = i;
@@ -536,16 +590,16 @@ void generate_header (Image::Header& header, DWI::Tractography::Reader& file, co
 
 
 
-Math::Matrix gen_interp_matrix (const size_t os_factor)
+Math::Matrix gen_interp_matrix (const unsigned int os_factor)
 {
   Math::Matrix M;
   if (os_factor > 1) {
-    const size_t dim = os_factor - 1;
+    const unsigned int dim = os_factor - 1;
     Hermite interp (HERMITE_TENSION);
     M.allocate (dim, 4);
-    for (size_t i = 0; i != dim; ++i) {
+    for (unsigned int i = 0; i != dim; ++i) {
       interp.set ((i + 1.0) / float(os_factor));
-      for (size_t j = 0; j != 4; ++j)
+      for (unsigned int j = 0; j != 4; ++j)
         M(i,j) = interp.coef (j);
     }
   }
@@ -560,7 +614,7 @@ void oversample_header (Image::Header& header, const std::vector<float>& voxel_s
 
   Math::Matrix M (header.transform());
 
-  for (size_t i = 0; i != 3; ++i) {
+  for (unsigned int i = 0; i != 3; ++i) {
     M(i, 3) += 0.5 * (voxel_size[i] - header.axes.vox[i]);
     header.axes.dim[i] = ceil (header.axes.dim[i] * header.axes.vox[i] / voxel_size[i]);
     header.axes.vox[i] = voxel_size[i];
@@ -585,6 +639,7 @@ EXECUTE {
 
   const bool colour         = get_options (2).size();
   const bool fibre_fraction = get_options (3).size();
+  const bool lstdi          = get_options (4).size();
 
   std::vector<float> voxel_size;
   std::vector<OptBase> opt = get_options(1);
@@ -614,11 +669,11 @@ EXECUTE {
     file.open (argument[0].get_string(), properties);
   }
 
-  opt = get_options (4);
+  opt = get_options (5);
   if (opt.size())
     header.data_type.parse (data_type_choices[opt[0][0].get_int()]);
   else
-    header.data_type = (fibre_fraction || colour) ? DataType::Float32 : DataType::UInt32;
+    header.data_type = (fibre_fraction || colour || lstdi) ? DataType::Float32 : DataType::UInt32;
 
   for (DWI::Tractography::Properties::const_iterator i = properties.begin(); i != properties.end(); ++i)
     header.comments.push_back (i->first + ": " + i->second);
@@ -632,7 +687,7 @@ EXECUTE {
   info ("intensity scaling factor set to " + str(scaling_factor));
 
   size_t resample_factor;
-  opt = get_options (5);
+  opt = get_options (6);
   if (opt.size()) {
     resample_factor = opt[0][0].get_int();
     info ("track interpolation factor manually set to " + str(resample_factor));
@@ -661,13 +716,13 @@ EXECUTE {
     header.axes.desc[3] = "direction";
     header.comments.push_back (std::string ("coloured track density map"));
 
-    Image::Position       pos    (*argument[1].get_image (header));
-    TrackMapper<VoxelDir> mapper (pos, interp_matrix);
-    MapWriterColour       writer (pos, scaling_factor);
+    Image::Position          pos    (*argument[1].get_image (header));
+    TrackMapper<SetVoxelDir> mapper (pos, interp_matrix);
+    MapWriterColour          writer (pos, scaling_factor, lstdi);
 
     ProgressBar::init (num_tracks, "mapping tracks to colour image... ");
     while (file.next (tck)) {
-      std::set<VoxelDir> mapped_voxels;
+      SetVoxelDir mapped_voxels;
       mapper.map (tck, mapped_voxels);
       writer.write (mapped_voxels);
       ProgressBar::inc();
@@ -678,19 +733,37 @@ EXECUTE {
   else {
 
     header.axes.set_ndim(3);
-    header.comments.push_back (std::string (("track ") + str(fibre_fraction ? "fraction" : "count") + " map"));
+    header.comments.push_back (std::string (("track ") + str(fibre_fraction ? "fraction" : "count") + " map" + str (lstdi ? ", scaled by inverse track length" : "")));
 
-    Image::Position    pos    (*argument[1].get_image(header));
-    TrackMapper<Voxel> mapper (pos, interp_matrix);
-    MapWriter          writer (pos, scaling_factor);
+    Image::Position       pos    (*argument[1].get_image(header));
+    TrackMapper<SetVoxel> mapper (pos, interp_matrix);
+
+    if (fibre_fraction || lstdi) {
+
+      MapWriter<float> writer (pos, scaling_factor, lstdi);
+
+      ProgressBar::init (num_tracks, "mapping tracks to image... ");
+      while (file.next (tck)) {
+        SetVoxel mapped_voxels;
+        mapper.map (tck, mapped_voxels);
+        writer.write (mapped_voxels);
+        ProgressBar::inc();
+      }
+
+    } else {
+
+      MapWriter<uint32_t> writer (pos, scaling_factor, lstdi);
+
+      ProgressBar::init (num_tracks, "mapping tracks to image... ");
+      while (file.next (tck)) {
+        SetVoxel mapped_voxels;
+        mapper.map (tck, mapped_voxels);
+        writer.write (mapped_voxels);
+        ProgressBar::inc();
+      }
 
-    ProgressBar::init (num_tracks, "mapping tracks to image... ");
-    while (file.next (tck)) {
-      std::set<Voxel> mapped_voxels;
-      mapper.map (tck, mapped_voxels);
-      writer.write (mapped_voxels);
-      ProgressBar::inc();
     }
+
     ProgressBar::done();
 
   }
