@@ -74,7 +74,7 @@
 
 #define MRTRIX_MAJOR_VERSION 0
 #define MRTRIX_MINOR_VERSION 2
-#define MRTRIX_MICRO_VERSION 9
+#define MRTRIX_MICRO_VERSION 10
 
 
 /** Prints the file and line number. Useful for debugging purposes. */
@@ -90,6 +90,22 @@
 #define BUSY_INTERVAL 0.1
 #define GUI_SPACING 5
 
+#ifdef __APPLE__
+# define CTRL_CMD_MASK GDK_MOD2_MASK
+#define MODIFIERS ( \
+        GDK_SHIFT_MASK | \
+        GDK_CONTROL_MASK | \
+        GDK_MOD2_MASK | \
+        GDK_BUTTON1_MASK | \
+        GDK_BUTTON2_MASK | \
+        GDK_BUTTON3_MASK | \
+        GDK_BUTTON4_MASK | \
+        GDK_BUTTON5_MASK | \
+        GDK_SUPER_MASK | \
+        GDK_HYPER_MASK | \
+        GDK_META_MASK )
+#else
+# define CTRL_CMD_MASK GDK_CONTROL_MASK
 #define MODIFIERS ( \
         GDK_SHIFT_MASK | \
         GDK_CONTROL_MASK | \
@@ -101,11 +117,25 @@
         GDK_SUPER_MASK | \
         GDK_HYPER_MASK | \
         GDK_META_MASK )
+#endif
 
 #define TMPFILE_ROOT "mrtrix-"
 #define TMPFILE_ROOT_LEN 7
 
 
+#ifdef G_OS_WIN32
+typedef struct __stat64 struct_stat64;
+# define STAT64 _stat64
+#else
+# ifdef __APPLE__
+typedef struct stat struct_stat64;
+#  define STAT64 stat
+# else
+typedef struct stat64 struct_stat64;
+#  define STAT64 stat64
+# endif
+#endif
+
 namespace std {
 
   template <class T> inline ostream& operator<< (ostream& stream, const vector<T>& V)
