@@ -97,10 +97,21 @@ namespace MR {
             S.image = roi->mask;
             S.focus = slice.focus;
 
-            if (!slice.orientation && slice.image->image->header().transform() == roi->mask->image->header().transform()) { 
-              S.projection = slice.projection;
-            }
+            bool transforms_differ = false;
+            if (slice.orientation) 
+              transforms_differ = true;
             else {
+              for (guint i = 0; i < 4; ++i) {
+                for (guint j = 0; j < 3; ++j) {
+                  if (fabs (slice.image->image->header().transform()(i,j) - roi->mask->image->header().transform()(i,j)) > 1e-5) {
+                    transforms_differ = true;
+                    break;
+                  }
+                }
+              }
+            }
+
+            if (transforms_differ) {
               const GLdouble* M (pane.get_modelview());
               float matrix[] = { 
                 M[0], M[1], M[2],
@@ -110,6 +121,9 @@ namespace MR {
               S.orientation.from_matrix (matrix);
               S.projection = 2;
             }
+            else 
+              S.projection = slice.projection;
+
             S.interpolate = false;
 
             Slice::Current current (S);
@@ -224,6 +238,7 @@ namespace MR {
           if (selection.size()) {
             MR::Image::Header header (*Window::Main->image->image);
             header.data_type = DataType::Bit;
+            header.axes.set_ndim (3);
             RefPtr<MR::Image::Object> obj (new MR::Image::Object);
             try {
               obj->create (selection[0], header); 
@@ -274,12 +289,15 @@ namespace MR {
       {
         Gtk::TreeModel::iterator iter = get_selection()->get_selected();
         if (!iter) return (false);
-        if ((event->state & MODIFIERS) != GDK_SHIFT_MASK && (event->state & MODIFIERS) != ( GDK_SHIFT_MASK | GDK_CONTROL_MASK )) return (false);
+	guint state = (event->state & MODIFIERS) & (~GDK_BUTTON1_MASK);
+        if (state != GDK_SHIFT_MASK && 
+            state != ( GDK_SHIFT_MASK | CTRL_CMD_MASK ))
+          return (false);
         row = *iter;
         bool show = row[columns.show];
         if (!show) return (false);
 
-        set = (event->state & MODIFIERS) == GDK_SHIFT_MASK;
+        set = state == GDK_SHIFT_MASK;
         editing = true;
 
         process (event->x, event->y);
