@@ -87,6 +87,7 @@ namespace MR {
         Glib::signal_timeout().connect (sigc::mem_fun (*this, &TrackList::on_refresh), 3000);
 
         set_tooltip_text ("right-click for more options");
+        set_tooltip_column (4);
       }
 
 
@@ -255,6 +256,7 @@ namespace MR {
         row[columns.pix] = colour_by_dir_pixbuf;
         row[columns.name] = Glib::path_get_basename (filename);
         row[columns.track] = track;
+        row[columns.tooltip_text] = filename + "\n\nright-click for more options";
 
         vertices.clear();
         get_selection()->select (row);
@@ -430,6 +432,7 @@ namespace MR {
         if (!parent.show()) return (true);
         Gtk::TreeModel::Children tracks = model->children();
         if (tracks.size() == 0) return (true);
+        std::vector<Gtk::TreeModel::Children::iterator> iterators_to_delete;
         for (Gtk::TreeModel::Children::iterator iter = tracks.begin(); iter != tracks.end(); ++iter) {
           bool show = (*iter)[columns.show];
           if (show) {
@@ -441,9 +444,18 @@ namespace MR {
                 Window::Main->update (&parent);
               }
             }
-            catch (...) { }
+            catch (...) { 
+              iterators_to_delete.push_back (iter);
+            }
           }
         }
+
+        for (size_t n = 0; n < iterators_to_delete.size(); ++n)
+          model->erase (iterators_to_delete[n]);
+
+        if (iterators_to_delete.size()) 
+          Window::Main->update (&parent);
+
         return (true);
       }
 
