@@ -37,6 +37,7 @@
 #include "mrview/dialog/properties.h" 
 #include "mrview/dialog/opengl.h" 
 
+
 namespace MR {
   namespace Viewer {
 
@@ -76,36 +77,129 @@ namespace MR {
       file_menu.items()[2].set_sensitive (false);
       file_menu.items()[4].set_sensitive (false);
 
+
       menubar.items().push_back (MenuElem ("_View"));
       menubar.items().back().set_submenu (view_menu);
-      view_menu.items().push_back (CheckMenuElem ("Side_bar", Gtk::AccelKey ("F9"), sigc::mem_fun(*this, &Window::on_view_sidebar)));
-      view_menu.items().push_back (CheckMenuElem ("_Interpolate", Gtk::AccelKey ("<control>I"), sigc::mem_fun (*this, &Window::on_view_interpolate)));
-      view_menu.items().push_back (CheckMenuElem ("_Lock to image axes", sigc::mem_fun (*this, &Window::on_view_lock_to_axes)));
+
+      view_menu.items().push_back (
+          CheckMenuElem ("Side_bar", 
+#ifdef __APPLE__
+            Gtk::AccelKey ("<control><shift>D"), 
+#else
+            Gtk::AccelKey ("F9"), 
+#endif
+            sigc::mem_fun(*this, &Window::on_view_sidebar)));
+
+      view_menu.items().push_back (
+          CheckMenuElem ("_Interpolate",
+            Gtk::AccelKey ("<control>I"), 
+            sigc::mem_fun (*this, &Window::on_view_interpolate)));
+
+      view_menu.items().push_back (
+          CheckMenuElem ("_Lock to image axes",
+            sigc::mem_fun (*this, &Window::on_view_lock_to_axes)));
+
       view_menu.items().push_back (MenuElem ("Colour _map"));
       view_menu.items().back().set_submenu (colourmap_menu);
-      view_menu.items().push_back (SeparatorElem());
-      view_menu.items().push_back (RadioMenuElem (projection_group, "_Axial", Gtk::AccelKey ("A"),sigc::mem_fun (*this, &Window::on_view_axial)));
-      view_menu.items().push_back (RadioMenuElem (projection_group, "_Sagittal", Gtk::AccelKey ("S"),sigc::mem_fun (*this, &Window::on_view_sagittal)));
-      view_menu.items().push_back (RadioMenuElem (projection_group, "_Coronal", Gtk::AccelKey ("C"),sigc::mem_fun (*this, &Window::on_view_coronal)));
-      view_menu.items().push_back (SeparatorElem());
-      view_menu.items().push_back (CheckMenuElem ("Show F_ocus", Gtk::AccelKey ("<control>F"), sigc::mem_fun (*this, &Window::on_view_focus)));
-      view_menu.items().push_back (MenuElem ("Reset _Windowing", Gtk::AccelKey ("<control>R"), sigc::mem_fun (*this, &Window::on_view_reset_windowing)));
-      view_menu.items().push_back (MenuElem ("_Reset View", Gtk::AccelKey ("<control><shift>R"), sigc::mem_fun (*this, &Window::on_view_reset)));
-      view_menu.items().push_back (CheckMenuElem ("_Full screen", Gtk::AccelKey ("F11"), sigc::mem_fun (*this, &Window::on_view_full_screen)));
 
-      colourmap_menu.items().push_back (RadioMenuElem (colourmap_group, "_Gray", sigc::bind<int> (sigc::mem_fun (*this, &Window::on_colourmap), 0)));
-      colourmap_menu.items().push_back (RadioMenuElem (colourmap_group, "_Hot", sigc::bind<int> (sigc::mem_fun (*this, &Window::on_colourmap), 1)));
-      colourmap_menu.items().push_back (RadioMenuElem (colourmap_group, "_Cool", sigc::bind<int> (sigc::mem_fun (*this, &Window::on_colourmap), 2)));
-      colourmap_menu.items().push_back (RadioMenuElem (colourmap_group, "_Jet", sigc::bind<int> (sigc::mem_fun (*this, &Window::on_colourmap), 3)));
+      view_menu.items().push_back (SeparatorElem());
+
+      view_menu.items().push_back (
+          RadioMenuElem (projection_group, "_Axial", 
+            Gtk::AccelKey ("A"),
+            sigc::mem_fun (*this, &Window::on_view_axial)));
+
+      view_menu.items().push_back (
+          RadioMenuElem (projection_group, "_Sagittal", 
+            Gtk::AccelKey ("S"),
+            sigc::mem_fun (*this, &Window::on_view_sagittal)));
+
+      view_menu.items().push_back (
+          RadioMenuElem (projection_group, "_Coronal",
+            Gtk::AccelKey ("C"),
+            sigc::mem_fun (*this, &Window::on_view_coronal)));
+
+      view_menu.items().push_back (SeparatorElem());
+
+      view_menu.items().push_back (
+          MenuElem ("Zoom in",
+            Gtk::AccelKey ("Home"),
+            sigc::mem_fun (*this, &Window::on_zoom_in)));
+
+      view_menu.items().push_back (
+          MenuElem ("Zoom out",
+            Gtk::AccelKey ("End"),
+            sigc::mem_fun (*this, &Window::on_zoom_out)));
+
+      view_menu.items().push_back (SeparatorElem());
+
+      view_menu.items().push_back (
+          CheckMenuElem ("Show F_ocus",
+            Gtk::AccelKey ("<control>F"),
+            sigc::mem_fun (*this, &Window::on_view_focus)));
+      dynamic_cast<Gtk::CheckMenuItem&> (view_menu.items().back()).set_active (true);
+
+      view_menu.items().push_back (
+          MenuElem ("Reset _Windowing",
+            Gtk::AccelKey ("<control>R"), 
+            sigc::mem_fun (*this, &Window::on_view_reset_windowing)));
+
+      view_menu.items().push_back (
+          MenuElem ("_Reset View", 
+            Gtk::AccelKey ("<control><shift>R"),
+            sigc::mem_fun (*this, &Window::on_view_reset)));
+
+      view_menu.items().push_back (
+          CheckMenuElem ("_Full screen", 
+#ifdef __APPLE__
+            Gtk::AccelKey ("<control><shift>F"), 
+#else
+            Gtk::AccelKey ("F11"),
+#endif
+            sigc::mem_fun (*this, &Window::on_view_full_screen)));
+
+
+      colourmap_menu.items().push_back (
+          RadioMenuElem (colourmap_group, "_Gray", 
+            sigc::bind<int> (sigc::mem_fun (*this, &Window::on_colourmap), 0)));
+
+      colourmap_menu.items().push_back (
+          RadioMenuElem (colourmap_group, "_Hot", 
+            sigc::bind<int> (sigc::mem_fun (*this, &Window::on_colourmap), 1)));
+
+      colourmap_menu.items().push_back (
+          RadioMenuElem (colourmap_group, "_Cool",
+            sigc::bind<int> (sigc::mem_fun (*this, &Window::on_colourmap), 2)));
+
+      colourmap_menu.items().push_back (
+          RadioMenuElem (colourmap_group, "_Jet",
+            sigc::bind<int> (sigc::mem_fun (*this, &Window::on_colourmap), 3)));
+
       colourmap_menu.items().push_back (SeparatorElem());
-      colourmap_menu.items().push_back (RadioMenuElem (colourmap_group, "_RGB", sigc::bind<int> (sigc::mem_fun (*this, &Window::on_colourmap), COLOURMAP_RGB)));
-      colourmap_menu.items().push_back (RadioMenuElem (colourmap_group, "Comple_x", sigc::bind<int> (sigc::mem_fun (*this, &Window::on_colourmap), COLOURMAP_COMPLEX)));
+
+      colourmap_menu.items().push_back (
+          RadioMenuElem (colourmap_group, "_RGB", 
+            sigc::bind<int> (sigc::mem_fun (*this, &Window::on_colourmap), COLOURMAP_RGB)));
+
+      colourmap_menu.items().push_back (
+          RadioMenuElem (colourmap_group, "Comple_x", 
+            sigc::bind<int> (sigc::mem_fun (*this, &Window::on_colourmap), COLOURMAP_COMPLEX)));
 
       menubar.items().push_back (MenuElem ("_Image"));
       menubar.items().back().set_submenu (image_menu);
-      image_menu.items().push_back (MenuElem ("_Next Image", Gtk::AccelKey ("<control>Page_Up"), sigc::mem_fun(*this, &Window::on_image_next)));
-      image_menu.items().push_back (MenuElem ("_Previous Image", Gtk::AccelKey ("<control>Page_Down"), sigc::mem_fun(*this, &Window::on_image_previous)));
+
+      image_menu.items().push_back (
+          MenuElem ("_Next Image", 
+            Gtk::AccelKey ("<control>Page_Up"),
+            sigc::mem_fun(*this, &Window::on_image_next)));
+
+      image_menu.items().push_back (
+          MenuElem ("_Previous Image",
+            Gtk::AccelKey ("<control>Page_Down"), 
+            sigc::mem_fun(*this, &Window::on_image_previous)));
+
       image_menu.items().push_back (SeparatorElem ());
+
       image_menu.items().push_back (MenuElem ("No image loaded"));
 
       image_menu.items()[0].set_sensitive (false);
@@ -114,13 +208,30 @@ namespace MR {
 
       menubar.items().push_back (MenuElem ("_Help"));
       menubar.items().back().set_submenu (help_menu);
-      help_menu.items().push_back (MenuElem ("_OpenGL Info", sigc::mem_fun(*this, &Window::on_help_OpenGL_info)));
-      help_menu.items().push_back (StockMenuElem (Gtk::Stock::ABOUT, sigc::mem_fun(*this, &Window::on_help_about)));
 
+      help_menu.items().push_back (
+          MenuElem ("_OpenGL Info",
+            sigc::mem_fun(*this, &Window::on_help_OpenGL_info)));
+
+      help_menu.items().push_back (
+          StockMenuElem (Gtk::Stock::ABOUT,
+            sigc::mem_fun(*this, &Window::on_help_about)));
+
+
+      eventbox.add (statusbar);
+      eventbox.drag_source_set (std::vector<Gtk::TargetEntry>());
+      eventbox.drag_source_add_text_targets();
+      eventbox.signal_drag_data_get().connect (sigc::mem_fun (*this, &Window::on_drag));
+      eventbox.set_tooltip_text ("Position of focus in real and voxel coordinates.\nUse drag & drop to copy onto the command-line");
 
       main_box.pack_start (menubar, Gtk::PACK_SHRINK);
       main_box.pack_start (paned);
-      main_box.pack_start (statusbar, Gtk::PACK_SHRINK);
+      main_box.pack_start (eventbox, Gtk::PACK_SHRINK);
+
+      std::list<Gtk::TargetEntry> list;
+      list.push_back (Gtk::TargetEntry("text/uri-list"));
+      drag_dest_set (list, Gtk::DEST_DEFAULT_MOTION | Gtk::DEST_DEFAULT_DROP, Gdk::ACTION_COPY | Gdk::ACTION_MOVE);
+      signal_drag_data_received().connect (sigc::mem_fun (*this, &Window::on_drop));
 
       paned.pack1 (display_area, true, false);
       paned.pack2 (sidebar, false, false);
@@ -129,11 +240,8 @@ namespace MR {
       sidebar.hide();
 
       signal_key_press_event().connect (sigc::mem_fun(display_area, &DisplayArea::on_key_press), false);
-      dynamic_cast<Gtk::CheckMenuItem&> (view_menu.items()[9]).set_active (true);
 
       realize();
-      for (guint n = 0; n < argument.size(); n++) manage (argument[n].get_image());
-
       error = Viewer::ErrorDialog::error;
       info = Viewer::ErrorDialog::info;
 
@@ -141,7 +249,11 @@ namespace MR {
       ProgressBar::display_func = ProgressDialog::display;
       ProgressBar::done_func = ProgressDialog::done;
 
-      if (images.size()) on_image_selected (images[0]);
+      std::vector<RefPtr<MR::Image::Object> > image_list;
+      for (guint n = 0; n < argument.size(); n++) 
+        image_list.push_back (argument[n].get_image());
+      load (image_list);
+
       on_view_reset();
     }
 
@@ -151,8 +263,6 @@ namespace MR {
     Window::~Window() { }
 
 
-
-
     void Window::manage (RefPtr<MR::Image::Object> obj)
     {
       using namespace Gtk::Menu_Helpers;
@@ -181,6 +291,21 @@ namespace MR {
 
 
 
+    void Window::load (std::vector<RefPtr<MR::Image::Object> >& image_list)
+    {
+      if (image_list.empty()) 
+        return;
+      int index = images.size();
+      for (guint n = 0; n < image_list.size(); n++) 
+        manage (image_list[n]);
+      dynamic_cast<Gtk::RadioMenuItem&> (image_menu.items()[3+index]).set_active (true);
+      on_image_selected (images[index]);
+    }
+
+
+
+
+
     void Window::update_statusbar ()
     {
       statusbar.pop ();
@@ -198,7 +323,7 @@ namespace MR {
         I.P (pix);
         value = I.value();
         pos = printf ("%.2f %.2f %.2f ", pix[0], pix[1], pix[2]);
-        for (int n = 3; n < I.ndim(); n++) pos += str (I.dim(n)) + " ";
+        for (int n = 3; n < I.ndim(); n++) pos += str (I[n]) + " ";
       }
       else {
         P.set (0, round (pix[0]));
@@ -222,13 +347,7 @@ namespace MR {
 
       if (dialog.run() == Gtk::RESPONSE_OK) {
         std::vector<RefPtr<MR::Image::Object> > selection = dialog.get_images();
-        if (selection.size()) {
-          int first = images.size();
-          for (guint n = 0; n < selection.size(); n++) 
-            manage (selection[n]);
-          dynamic_cast<Gtk::RadioMenuItem&> (image_menu.items()[3+first]).set_active (true);
-          on_image_selected (images[first]);
-        }
+        load (selection);
       }
     }
 
@@ -358,6 +477,21 @@ namespace MR {
     }
 
 
+
+    void Window::on_zoom_in () 
+    {
+      Slice::Current S (pane());
+      if (!S.image) return;
+      zoom (-1);
+    }
+
+    void Window::on_zoom_out () 
+    {
+      Slice::Current S (pane());
+      if (!S.image) return;
+      zoom (1);
+    }
+
     void Window::on_view_interpolate () 
     {
       Slice::Current S (pane());
@@ -421,7 +555,7 @@ namespace MR {
       }
       S.focus.invalidate();
       pane().FOV = GSL_NAN;
-      const MR::Image::Object& ima (*S.image->image);
+      const Image& ima (*S.image);
       S.projection = minindex (ima.dim(0)*ima.vox(0), ima.dim(1)*ima.vox(1), ima.dim(2)*ima.vox(2));
       update_projection();
       update();
@@ -453,6 +587,8 @@ namespace MR {
         set_title (image->image->name());
         dynamic_cast<Gtk::CheckMenuItem&> (view_menu.items()[1]).set_active (S.interpolate);
         dynamic_cast<Gtk::CheckMenuItem&> (view_menu.items()[2]).set_active (!S.orientation);
+        int entry = S.colourmap < COLOURMAP_RGB ? S.colourmap : (S.colourmap-COLOURMAP_RGB+COLOURMAP_MAX_SCALAR_INDEX+1);
+        dynamic_cast<Gtk::CheckMenuItem&> (colourmap_menu.items()[entry]).set_active();
       }
       else set_title ("MRView");
       update();
@@ -541,10 +677,49 @@ namespace MR {
 
 
 
+    void Window::on_drag (const Glib::RefPtr<Gdk::DragContext>& context, Gtk::SelectionData& selection_data, guint info, guint time)
+    {
+      const Slice::Current S (pane());
+      if (!S.image) return;
+      if (!S.focus) return;
+      String text (printf ("%.2f,%.2f,%.2f", S.focus[0], S.focus[1], S.focus[2]));
+      selection_data.set (selection_data.get_target(), 8, (const guint8*) text.c_str(), text.size());
+    }
 
 
 
-    bool Window::show_focus () const { return (dynamic_cast<Gtk::CheckMenuItem&> (view_menu.items()[9]).get_active()); }
+
+    void Window::on_drop (const Glib::RefPtr<Gdk::DragContext>& context, int x, int y, const Gtk::SelectionData& selection_data, guint info, guint time)
+    {
+      if (selection_data.get_length() && selection_data.get_format() == 8) {
+        std::vector<Glib::ustring> file_list (selection_data.get_uris());
+
+        if (file_list.size()) {
+          std::vector<RefPtr<MR::Image::Object> > image_list;
+
+          for (guint n = 0; n < file_list.size(); ++n) {
+            String path = Glib::filename_from_uri (file_list[n]);
+            try {
+              RefPtr<MR::Image::Object> obj (new MR::Image::Object);
+              obj->open (path);
+              image_list.push_back (obj);
+            }
+            catch (...) {
+              error ("error opening file \"" + path + "\"");
+            }
+          }
+
+          load (image_list);
+          context->drag_finish (true, false, time);
+          return;
+        }
+      }
+      context->drag_finish (false, false, time);
+    }
+
+
+
+    bool Window::show_focus () const { return (dynamic_cast<Gtk::CheckMenuItem&> (view_menu.items()[12]).get_active()); }
 
 
   }
